@@ -4,11 +4,15 @@
 # directory. If it is already there, merge in new directives. Called with
 # <osname> <osversion> <install dir> <config dir> <module>+
 
-@ARGV >= 4 || die "usage: copyconfig.pl <os> <version> <webmin-dir> <config-dir> [module ...]";
+@ARGV >= 4 || die "usage: copyconfig.pl <os>[/real-os] <version>[/real-version] <webmin-dir> <config-dir> [module ...]";
 $os = $ARGV[0];
 $ver = $ARGV[1];
 $wadir = $ARGV[2];
 $confdir = $ARGV[3];
+($os, $real_os) = split(/\//, $os);
+($ver, $real_ver) = split(/\//, $ver);
+$real_os =~ s/ /-/g;
+$real_ver =~ s/ /-/g;
 
 # Find all clones
 opendir(DIR, $wadir);
@@ -23,28 +27,48 @@ closedir(DIR);
 # For each module, copy its config to itself and all clones
 @mods = @ARGV[4..$#ARGV];
 foreach $m (@mods) {
-	# Find any range-number config files
+	# Find any range-number config files. Search first by real OS type
+	# (ie Ubuntu 6.1), then by internal OS code (ie. debian-linux 3.1)
 	$srcdir = "$wadir/$m";
-	$rangefile = undef;
-	opendir(DIR, $srcdir);
-	while($f = readdir(DIR)) {
-		if ($f =~ /^config\-\Q$os\E\-([0-9\.]+)\-([0-9\.]+)$/ &&
-		    $ver >= $1 && $ver <= $2) {
-			$rangefile = "$srcdir/$f";
-			}
-		elsif ($f =~ /^config\-\Q$os\E\-([0-9\.]+)\-(\*|ALL)$/ &&
-		       $ver >= $1) {
-			$rangefile = "$srcdir/$f";
-			}
-		elsif ($f =~ /^config\-\Q$os\E\-(\*|ALL)\-([0-9\.]+)$/ &&
-		       $ver <= $2) {
-			$rangefile = "$srcdir/$f";
+	$rangefile = $real_rangefile = undef;
+	foreach $ov ([ $real_os, $real_ver, \$real_rangefile ],
+		     [ $os, $ver, \$rangefile ]) {
+		my ($o, $v, $rf) = @$ov;
+		opendir(DIR, $srcdir);
+		while($f = readdir(DIR)) {
+			if ($f =~ /^config\-\Q$o\E\-([0-9\.]+)\-([0-9\.]+)$/ &&
+			    $v >= $1 && $v <= $2) {
+				$$rf = "$srcdir/$f";
+				}
+			elsif ($f =~ /^config\-\Q$o\E\-([0-9\.]+)\-(\*|ALL)$/ &&
+			       $v >= $1) {
+				$$rf = "$srcdir/$f";
+				}
+			elsif ($f =~ /^config\-\Q$o\E\-(\*|ALL)\-([0-9\.]+)$/ &&
+			       $v <= $2) {
+				$$rf = "$srcdir/$f";
+				}
 			}
+		closedir(DIR);
 		}
-	closedir(DIR);
 
-	# Find the best-matching config file
-	if (-r "$srcdir/config-$os-$ver") {
+	# Find the best-matching config file. Search first by real OS type,
+	# then by internal OS code
+
+	# Check for real OS match by name and version, version range, or
+	# name only
+	if (-r "$srcdir/config-$real_os-$real_ver") {
+		$conf = "$srcdir/config-$real_os-$real_ver";
+		}
+	elsif ($real_rangefile) {
+		$conf = $real_rangefile;
+		}
+	elsif (-r "$srcdir/config-$real_os") {
+		$conf = "$srcdir/config-$real_os";
+		}
+
+	# Check for OS code match by name and version, version range, or name
+	elsif (-r "$srcdir/config-$os-$ver") {
 		$conf = "$srcdir/config-$os-$ver";
 		}
 	elsif ($rangefile) {
@@ -53,12 +77,16 @@ foreach $m (@mods) {
 	elsif (-r "$srcdir/config-$os") {
 		$conf = "$srcdir/config-$os";
 		}
+
+	# Check for config for an entire OS class, like *-linux
 	elsif ($os =~ /^(\S+)-(\S+)$/ && -r "$srcdir/config-ALL-$2") {
 		$conf = "$srcdir/config-ALL-$2";
 		}
 	elsif ($os =~ /^(\S+)-(\S+)$/ && -r "$srcdir/config-*-$2") {
 		$conf = "$srcdir/config-*-$2";
 		}
+
+	# Use default config file, if it exists
 	elsif (-r "$srcdir/config") {
 		$conf = "$srcdir/config";
 		}
@@ -83,7 +111,6 @@ foreach $m (@mods) {
 		&write_file("$confdir/$c/config", \%newconf);
 		}
 	}
-
 print join(" ", @newmods),"\n";
 
 # read_file(file, array)
@@ -92,7 +119,7 @@ sub read_file
 {
 local($arr);
 $arr = $_[1];
-open(ARFILE, $_[0]) || return 0;
+open(ARFILE, "<".$_[0]) || return 0;
 while(<ARFILE>) {
 	s/\r|\n//g;
         if (!/^#/ && /^([^=]+)=(.*)$/) { $$arr{$1} = $2; }
@@ -107,7 +134,7 @@ sub write_file
 {
 local($arr);
 $arr = $_[1];
-open(ARFILE, "> $_[0]");
+open(ARFILE, ">".$_[0]);
 foreach $k (keys %$arr) {
         print ARFILE "$k=$$arr{$k}\n";
         }
