@@ -78,6 +78,11 @@ if [ "$allmods" = "" ]; then
 fi
 echo ""
 
+# Load package-defined variable overrides
+if [ -r "$srcdir/setup-pre.sh" ]; then
+        . "$srcdir/setup-pre.sh"
+fi
+
 # Ask for usermin config directory
 echo "***********************************************************************"
 echo "Usermin uses separate directories for configuration files and log files."
@@ -162,10 +167,11 @@ if [ "$upgrading" = 1 ]; then
 	$perl "$wadir/newmods.pl" $config_dir $allmods
 
 	# Update miniserv.conf with new root directory and mime types file
-	grep -v "^root=" $config_dir/miniserv.conf | grep -v "^mimetypes=" >$tempdir/$$.miniserv.conf
-	mv $tempdir/$$.miniserv.conf $config_dir/miniserv.conf
+	grep -v "^root=" $config_dir/miniserv.conf | grep -v "^mimetypes=" | grep -v "^server=" >"$tempdir/$$.miniserv.conf"
+	mv "$tempdir/$$.miniserv.conf" "$config_dir/miniserv.conf"
 	echo "root=$wadir" >> $config_dir/miniserv.conf
 	echo "mimetypes=$wadir/mime.types" >> $config_dir/miniserv.conf
+	echo "server=MiniServ/$ver" >> $config_dir/miniserv.conf
 	grep logout= $config_dir/miniserv.conf >/dev/null
 	if [ $? != "0" ]; then
 		echo "logout=$config_dir/logout-flag" >> $config_dir/miniserv.conf
@@ -267,10 +273,10 @@ else
 		echo ""
 		exit 6
 	fi
-	$perl -e 'exit ($] < 5.002 ? 1 : 0)'
+	$perl -e 'exit ($] < 5.008 ? 1 : 0)'
 	if [ $? = "1" ]; then
 		echo "ERROR: Detected old perl version. Usermin requires"
-		echo "perl 5.002 or better to run"
+		echo "perl 5.8 or better to run"
 		echo ""
 		exit 7
 	fi
@@ -282,6 +288,9 @@ else
 		exit 8
 	fi
 	$perl -e '$c = crypt("xx", "yy"); exit($c ? 0 : 1)'
+        if [ $? != "0" ]; then
+                $perl -e 'use Crypt::UnixCrypt' >/dev/null 2>&1
+        fi
 	if [ $? != "0" ]; then
 		echo "ERROR: Perl crypt function does not work. Maybe Perl has"
 		echo "not been properly installed on your system"
@@ -305,12 +314,12 @@ else
 		if [ "$autoos" = "" ]; then
 			autoos=2
 		fi
-		$perl "$srcdir/oschooser.pl" "$srcdir/os_list.txt" $tempdir/$$.os $autoos
+		$perl "$srcdir/oschooser.pl" "$srcdir/os_list.txt" "$tempdir/$$.os" $autoos
 		if [ $? != 0 ]; then
 			exit $?
 		fi
-		. $tempdir/$$.os
-		rm -f $tempdir/$$.os
+		. "$tempdir/$$.os"
+		rm -f "$tempdir/$$.os"
 	fi
 	echo "Operating system name:    $real_os_type"
 	echo "Operating system version: $real_os_version"
@@ -342,11 +351,13 @@ else
 		echo ""
 		exit 12
 	fi
-	$perl -e 'use Socket; socket(FOO, PF_INET, SOCK_STREAM, getprotobyname("tcp")); setsockopt(FOO, SOL_SOCKET, SO_REUSEADDR, pack("l", 1)); bind(FOO, pack_sockaddr_in($ARGV[0], INADDR_ANY)) || exit(1); exit(0);' $port
-	if [ $? != "0" ]; then
-		echo "ERROR: TCP port $port is already in use by another program"
-		echo ""
-		exit 13
+	if [ "$noportcheck" = "" ]; then
+		$perl -e 'use Socket; socket(FOO, PF_INET, SOCK_STREAM, getprotobyname("tcp")); setsockopt(FOO, SOL_SOCKET, SO_REUSEADDR, pack("l", 1)); bind(FOO, pack_sockaddr_in($ARGV[0], INADDR_ANY)) || exit(1); exit(0);' $port
+		if [ $? != "0" ]; then
+			echo "ERROR: TCP port $port is already in use by another program"
+			echo ""
+			exit 13
+		fi
 	fi
 
 	# Ask the user if SSL should be used
@@ -398,6 +409,13 @@ else
 	echo "logtime=168" >> $cfile
 	echo "ppath=$ppath" >> $cfile
 	echo "ssl=$ssl" >> $cfile
+	echo "no_ssl2=1" >> $cfile
+	echo "no_ssl3=1" >> $cfile
+	openssl version 2>&1 | grep "OpenSSL 1" 2>&1 >/dev/null
+	if [ "$?" == "0" ]; then
+		echo "no_tls1=1" >> $cfile
+		echo "no_tls1_1=1" >> $cfile    
+	fi
 	echo "env_WEBMIN_CONFIG=$config_dir" >> $cfile
 	echo "env_WEBMIN_VAR=$var_dir" >> $cfile
 	echo "atboot=$atboot" >> $cfile
@@ -416,9 +434,14 @@ else
 		echo "session=1" >> $cfile
 	fi
 	echo "unixauth=user" >> $cfile
-	echo "localauth=1" >> $cfile
 	echo "pam=usermin" >> $cfile
-	#echo "denyusers=root" >> $cfile
+	echo premodules=WebminCore >> $cfile
+	echo "server=MiniServ/$ver" >> $cfile
+
+	# Append package-specific info to config file
+	if [ -r "$wadir/miniserv-conf" ]; then
+		cat "$wadir/miniserv-conf" >>$cfile
+	fi
 
 	ufile=$config_dir/miniserv.users
 	echo "user:x:0::" > $ufile
@@ -430,7 +453,7 @@ else
 	if [ "$?" = "0" ]; then
 		# We can generate a new SSL key for this host
 		host=`hostname`
-		openssl req -newkey rsa:512 -x509 -nodes -out $tempdir/cert -keyout $tempdir/key -days 1825 >/dev/null 2>&1 <<EOF
+		openssl req -newkey rsa:2048 -x509 -nodes -out "$tempdir/cert" -keyout "$tempdir/key" -days 1825 -sha256 >/dev/null 2>&1 <<EOF
 .
 .
 .
@@ -440,9 +463,9 @@ Usermin Webserver on $host
 root@$host
 EOF
 		if [ "$?" = "0" ]; then
-			cat $tempdir/cert $tempdir/key >$kfile
+			cat "$tempdir/cert" "$tempdir/key" >$kfile
 		fi
-		rm -f $tempdir/cert $tempdir/key
+		rm -f "$tempdir/cert" "$tempdir/key"
 	fi
 	if [ ! -r $kfile ]; then
 		# Fall back to the built-in key
@@ -456,9 +479,13 @@ EOF
 	echo ""
 
 	echo "Creating access control file.."
+	defmods=`cat "$wadir/defaultmodules" 2>/dev/null`
+	if [ "$defmods" = "" ]; then
+		defmods="$allmods"
+	fi
 	afile=$config_dir/webmin.acl
 	rm -f $afile
-	echo "user: $allmods" >> $afile
+	echo "user: $defmods" >> $afile
 	chmod 644 $afile
 	echo "..done"
 	echo ""
@@ -514,25 +541,22 @@ if [ "$upgrading" != 1 ]; then
 	echo "os_version=$os_version" >> $config_dir/config
 	echo "real_os_type=$real_os_type" >> $config_dir/config
 	echo "real_os_version=$real_os_version" >> $config_dir/config
-	if [ -r /etc/system.cnf ]; then
-		# Found a caldera system config file .. get the language
-		source /etc/system.cnf
-		if [ "$CONF_LST_LANG" = "us" ]; then
-			CONF_LST_LANG=en
-		elif [ "$CONF_LST_LANG" = "uk" ]; then
-			CONF_LST_LANG=en
-		fi
-		grep "lang=$CONF_LST_LANG," "$wadir/lang_list.txt" >/dev/null 2>&1
-		if [ "$?" = 0 ]; then
-			echo "lang=$CONF_LST_LANG" >> $config_dir/config
-		fi
-	fi
+	echo "lang=en.UTF-8" >> $config_dir/config
 
 	# Set additional usermin-specific options
 	echo "userconfig=.usermin" >> $config_dir/config
 	echo "overlang=ulang" >> $config_dir/config
 	echo "nofeedbackcc=2" >> $config_dir/config
 	echo "nofeedbackconf=1" >> $config_dir/config
+
+	# Disallow unknown referers by default
+	echo "referers_none=1" >> $config_dir/config
+else
+	# Disallow unknown referers if not set
+	grep referers_none= $config_dir/config >/dev/null
+	if [ "$?" != "0" ]; then
+		echo "referers_none=1" >> $config_dir/config
+	fi
 fi
 echo $ver > $config_dir/version
 echo "..done"
@@ -602,7 +626,7 @@ fi
 
 echo "Changing ownership and permissions .."
 chown -R root:bin $config_dir
-chmod -R 755 $config_dir
+find $config_dir -type d -exec chmod 755 {} +
 if [ "$nochown" = "" ]; then
 	chown -R root:bin "$wadir"
 	chmod -R og-w "$wadir"
@@ -623,6 +647,11 @@ else
 	rm -f $config_dir/install-dir
 fi
 
+# Run package-defined post-install script
+if [ -r "$srcdir/setup-post.sh" ]; then
+	. "$srcdir/setup-post.sh"
+fi
+
 if [ "$nostart" = "" ]; then
 	if [ "$inetd" != "1" ]; then
 		echo "Attempting to start Usermin mini web server.."
